<?php

/**
 * Returns possible filter values for progress entries
 *
 * @param string $strFieldValue
 * @param string $strFieldLabel
 * @param mixed $args (optional) Possible keys: user_id, attempt_id, success_status, completion_status, lesson_status, module_id
 *
 * @return array|null
 */
function cluevo_get_progress_filter_content($strFieldValue, $strFieldLabel, $args = [])
{
  global $wpdb;
  $progressTable = $wpdb->prefix . CLUEVO_DB_TABLE_MODULES_PROGRESS;
  $userTable = $wpdb->users;
  $moduleTable = $wpdb->prefix . CLUEVO_DB_TABLE_MODULES;

  if (strpos($strFieldLabel, ".") === false)
    $strFieldLabel = "p.$strFieldLabel";

  $sql = "SELECT  p.$strFieldValue AS value, $strFieldLabel AS label
      FROM $progressTable p
      INNER JOIN $userTable u
        ON p.user_id = u.ID
      INNER JOIN $moduleTable m
        ON p.module_id = m.module_id";

  $valid = ["user_id", "attempt_id", "success_status", "completion_status", "lesson_status", "module_id"];
  $where = [];
  $parms = [];

  foreach ($args as $arg => $value) {
    if ($arg == $strFieldValue)
      continue;

    if (in_array($arg, $valid) && (!empty($value) || ($arg == "attempt_id" && $value > -1))) {
      if (empty($parms))
        $sql .= " WHERE ";

      if (strpos($arg, ".") === false)
        $where[] = "p.$arg = %s";
      else
        $where[] = "$arg = %s";
      $parms[] = $value;
    }
  }

  $sql .= implode(" AND ", $where);
  $sql .= " GROUP BY p.$strFieldValue";

  if (!empty($parms)) {
    return $wpdb->get_results(
      $wpdb->prepare($sql, $parms)
    );
  }

  return $wpdb->get_results($sql);
}

/**
 * Returns an array with pagination information
 *
 * @param mixed $args (optional) Possible keys: module_id, user_id, attempt_id, success_status, completion_status, lesson_status
 * @param int $intPerPage (optional)
 */
function cluevo_get_progress_pagination($args = [], $intPerPage = 100)
{
  global $wpdb;
  $progressTable = $wpdb->prefix . CLUEVO_DB_TABLE_MODULES_PROGRESS;
  $userTable = $wpdb->users;
  $moduleTable = $wpdb->prefix . CLUEVO_DB_TABLE_MODULES;

  $sqlFields = "COUNT(*)";
  $sql = "SELECT $sqlFields
      FROM $progressTable p
      INNER JOIN $moduleTable m
        ON p.module_id = m.module_id
      INNER JOIN $userTable u
        ON p.user_id = u.ID";

  $valid = ["module_id", "user_id", "attempt_id", "success_status", "completion_status", "lesson_status"];
  $where = [];
  $parms = [];
  foreach ($args as $arg => $value) {
    if (in_array($arg, $valid) && (!empty($value) || ($arg == "attempt_id" && $value > -1))) {
      if (empty($parms))
        $sql .= " WHERE ";
      $where[] = "p.$arg = %s";
      $parms[] = $value;
    }
  }

  $sql .= implode(" AND ", $where);

  if (!empty($parms)) {
    $rows = $wpdb->get_var(
      $wpdb->prepare($sql, $parms)
    );
  } else {
    $rows = $wpdb->get_var($sql);
  }

  $pages = ceil($rows / $intPerPage);

  return ["pages" => $pages, "items_per_page" => $intPerPage, "items" => $rows];
}
/**
 * Retrieves scorm parameters from the database
 *
 * @param array $args (optional) Possible keys: module_id, user_id, attempt_id, parameter
 * @param int $intPage (optional)
 * @param int $intPerPage (optional)
 *
 * @return array|null
 */
function cluevo_get_parameters($args = [], $intPage = 0, $intPerPage = null)
{ //, $args = [], $intModuleId = null, $intUserId = null, $intAttemptId = null) {
  global $wpdb;
  $parmTable = $wpdb->prefix . CLUEVO_DB_TABLE_MODULE_PARMS;
  $userTable = $wpdb->users;
  $moduleTable = $wpdb->prefix . CLUEVO_DB_TABLE_MODULES;

  $sqlFields = "p.*, LEFT(p.value, 50) AS value, m.module_name, u.display_name, u.user_nicename, u.user_email, user_login";
  $sql = "SELECT $sqlFields
      FROM $parmTable p
      INNER JOIN $moduleTable m
        ON p.module_id = m.module_id
      INNER JOIN $userTable u
        ON p.user_id = u.ID";

  $valid = ["module_id", "user_id", "attempt_id", "parameter"];
  $where = [];
  $parms = [];
  foreach ($args as $arg => $value) {
    if (in_array($arg, $valid) && (!empty($value) || ($arg == "attempt_id" && $value > -1))) {
      if (empty($parms))
        $sql .= " WHERE ";

      $where[] = "p.$arg = %s";
      $parms[] = $value;
    }
  }

  $sql .= implode(" AND ", $where);
  $sql .= " ORDER BY p.id ASC";

  if (!empty($intPerPage)) {
    $limit = $intPage * $intPerPage;
    $sql .= " LIMIT $limit, $intPerPage";
  }

  if (!empty($parms)) {
    $result = $wpdb->get_results(
      $wpdb->prepare($sql, $parms)
    );
  } else {
    $result = $wpdb->get_results($sql);
  }

  return $result;
}

/**
 * Retrieves a list of available parameters
 *
 * @param mixed $args (optional) Possible keys: module_id, user_id, attempt_id
 *
 * @return array|null
 */
function cluevo_get_available_parameters($args = [])
{
  global $wpdb;
  $parmTable = $wpdb->prefix . CLUEVO_DB_TABLE_MODULE_PARMS;
  $userTable = $wpdb->users;
  $moduleTable = $wpdb->prefix . CLUEVO_DB_TABLE_MODULES;

  $sqlFields = "p.parameter";
  $sql = "SELECT $sqlFields
      FROM $parmTable p
      INNER JOIN $moduleTable m
        ON p.module_id = m.module_id
      INNER JOIN $userTable u
        ON p.user_id = u.ID";

  $valid = ["module_id", "user_id", "attempt_id"];
  $where = [];
  $parms = [];
  foreach ($args as $arg => $value) {
    if (in_array($arg, $valid) && (!empty($value) || ($arg == "attempt_id" && $value > -1))) {
      if (empty($parms))
        $sql .= " WHERE ";

      $where[] = "p.$arg = %d";
      $parms[] = $value;
    }
  }

  $sql .= implode(" AND ", $where);
  $sql .= " GROUP BY p.parameter";
  $sql .= " ORDER BY p.parameter ASC";

  if (!empty($parms)) {
    $result = $wpdb->get_results(
      $wpdb->prepare($sql, $parms)
    );
  } else {
    $result = $wpdb->get_results($sql);
  }

  return $result;
}

/**
 * Returns parameter pagination information for the given arguments
 *
 * @param mixed $args (optional) Possible keys: module_id, user_id, attempt_id, parameter
 * @param int $intPerPage (optional)
 *
 * @return array
 */
function cluevo_get_parameter_pagination($args = [], $intPerPage = 100)
{
  global $wpdb;
  $parmTable = $wpdb->prefix . CLUEVO_DB_TABLE_MODULE_PARMS;
  $userTable = $wpdb->users;
  $moduleTable = $wpdb->prefix . CLUEVO_DB_TABLE_MODULES;

  $sqlFields = "COUNT(*)";
  $sql = "SELECT $sqlFields
      FROM $parmTable p
      INNER JOIN $moduleTable m
        ON p.module_id = m.module_id
      INNER JOIN $userTable u
        ON p.user_id = u.ID";


  $valid = ["module_id", "user_id", "attempt_id", "parameter"];
  $where = [];
  $parms = [];
  foreach ($args as $arg => $value) {
    if (in_array($arg, $valid) && (!empty($value) || ($arg == "attempt_id" && $value > -1))) {
      if (empty($parms))
        $sql .= " WHERE ";
      $where[] = "p.$arg = %s";
      $parms[] = $value;
    }
  }

  $sql .= implode(" AND ", $where);

  if (!empty($parms)) {
    $rows = $wpdb->get_var(
      $wpdb->prepare($sql, $parms)
    );
  } else {
    $rows = $wpdb->get_var($sql);
  }

  $pages = ceil($rows / $intPerPage);

  return ["pages" => $pages, "items_per_page" => $intPerPage, "items" => $rows];
}

/**
 * Retrieves a list of possible filter values for parameters
 *
 * @param mixed $strFieldValue Column in the database for values
 * @param mixed $strFieldLabelColumn in the database to use for the label
 * @param mixed $args (optional) Possible keys: user_id, attempt_id, success_status, completion_status, lesson_status, module_id
 *
 * @return array|null
 */
function cluevo_get_parameters_filter_content($strFieldValue, $strFieldLabel, $args = [])
{
  global $wpdb;
  $parmTable = $wpdb->prefix . CLUEVO_DB_TABLE_MODULE_PARMS;
  $userTable = $wpdb->users;
  $moduleTable = $wpdb->prefix . CLUEVO_DB_TABLE_MODULES;

  if (strpos($strFieldLabel, ".") === false)
    $strFieldLabel = "p.$strFieldLabel";

  $sql = "SELECT  p.$strFieldValue AS value, $strFieldLabel AS label
      FROM $parmTable p
      INNER JOIN $userTable u
        ON p.user_id = u.ID
      INNER JOIN $moduleTable m
        ON p.module_id = m.module_id
      WHERE ";

  $valid = ["user_id", "attempt_id", "success_status", "completion_status", "lesson_status", "module_id"];
  $where = ["m.type_id = " . CLUEVO_SCORM_MODULE_TYPE_ID];
  $parms = [];

  foreach ($args as $arg => $value) {
    if ($arg == $strFieldValue)
      continue;

    if (in_array($arg, $valid) && (!empty($value) || ($arg == "attempt_id" && $value > -1))) {

      if (strpos($arg, ".") === false)
        $where[] = "p.$arg = %s";
      else
        $where[] = "$arg = %s";
      $parms[] = $value;
    }
  }

  $sql .= implode(" AND ", $where);
  $sql .= " GROUP BY p.$strFieldValue";

  if (!empty($parms)) {
    return $wpdb->get_results(
      $wpdb->prepare($sql, $parms)
    );
  }

  return $wpdb->get_results($sql);
}

function cluevo_render_reports_page()
{
  $active_tab = (!empty($_GET["tab"]) && ctype_alpha($_GET["tab"])) ? cluevo_strip_non_alphanum_dash($_GET["tab"]) : CLUEVO_ADMIN_TAB_REPORTS_MAIN;
  do_action('cluevo_init_admin_page');
?>
  <div class="cluevo-admin-page-container">
    <div class="cluevo-admin-page-title-container">
      <h1><?php esc_html_e("Reporting", "cluevo"); ?></h1>
      <img class="plugin-logo" src="<?php echo esc_url(plugins_url("/assets/logo-white.png", plugin_dir_path(__FILE__)), ['http', 'https']); ?>" />
    </div>
    <div class="cluevo-admin-page-content-container">
      <h2 class="nav-tab-wrapper cluevo">
        <?php do_action('cluevo_output_reports_tabs'); ?>
      </h2>
      <?php
      if (!empty($active_tab)) {
        if (has_action("cluevo_render_report_tab_{$active_tab}")) {
          do_action("cluevo_render_report_tab_{$active_tab}");
        } else {
          cluevo_render_reports_tab_reports();
        }
      } else {
        cluevo_render_reports_tab_reports();
      }
      ?>
    </div>
    <?php
  }

  function cluevo_render_reports_tab_reports()
  {
    wp_register_script('cluevo-admin-table-filter', plugins_url('/js/admin-table-filter.js', plugin_dir_path(__FILE__)), array(), CLUEVO_VERSION, true);
    wp_enqueue_script("cluevo-admin-table-filter");
    $items = null;
    $pagination = null;
    $itemId = (!empty($_GET["item"]) && is_numeric($_GET["item"])) ? (int)$_GET["item"] : null;
    $moduleId = (!empty($_GET["module"]) && is_numeric($_GET["module"])) ? (int)$_GET["module"] : null;
    $page = (isset($_GET["cur-page"]) && is_numeric($_GET["cur-page"]) && (int)$_GET["cur-page"] >= 0) ? (int)$_GET["cur-page"] : 0;
    $perPage = 10;
    $active_tab = (!empty($_GET["tab"]) && ctype_alpha($_GET["tab"])) ? cluevo_strip_non_alpha($_GET["tab"]) : CLUEVO_ADMIN_TAB_REPORTS_MAIN;
    $active_page = CLUEVO_ADMIN_PAGE_REPORTS;

    if (!empty($moduleId)) {
      //FIXME: it's completely random that this works because the parameters are shared between the two pages. It's probably better to link directly to the page instead of rendering it here.
      cluevo_render_reports_tab_progress(null);
    } else {
      $pagination = cluevo_get_learning_structure_item_children_pagination($itemId, $perPage);
      $items = cluevo_get_learning_structure_item_children($itemId, null, $page, $perPage);
      $nextPage = ($page + 1 < $pagination["pages"]) ? $page + 1 : null;

      foreach ($items as $key => $item) {
        $modules = $item->modules;
        if (is_object($item->modules)) {
          if (!empty($item->modules->id)) {
            if (!in_array($item->modules->id, $modules)) {
              $modules = $item->modules->id;
            }
          } else {
            $modules = [];
          }
        } else {
          if (!empty($item->module) && $item->module > 0) {
            if (!in_array($item->module, $modules)) {
              $modules[] = $item->module;
            }
          }
        }
        if (!empty($modules)) {
          $details = cluevo_get_progress_summary_for_modules($modules);
          $items[$key]->details = $details;
        }
      }

      $parentId = null;
      if (!empty($itemId)) {
        $parent = cluevo_get_learning_structure_item($itemId);
        $parentId = $parent->parent_id;
      }

      global $wpdb;
      $totalUsers = $wpdb->get_var("SELECT COUNT(*) FROM " . $wpdb->users);
    ?>
      <h1><?php echo esc_html($pagination["items"]) . " " . esc_html__("records found", "cluevo"); ?></h1>
      <?php if (!empty($items)) { ?>
        <?php if ($parentId !== null) { ?>
          <a href="<?php echo esc_url(admin_url("admin.php?page=$active_page&tab=$active_tab&item=$parentId"), ['http', 'https']); ?>" class="button">⮤ <?php echo esc_html($parent->name); ?></a>
        <?php } ?>
        <form action="<?php echo esc_url(admin_url("admin.php?page=$active_page"), ['http', 'https']); ?>">
          <input type="hidden" name="page" value="<?php echo esc_attr($active_page); ?>" />
          <input type="hidden" name="tab" value="<?php echo esc_attr($active_tab); ?>" />
          <input type="hidden" name="item" id="item" value="<?php echo esc_attr($itemId); ?>" />
          <table class="cluevo-admin-table cluevo-filtered-table">
            <tr>
              <th>#</th>
              <th class="left"><?php esc_html_e("Type", "cluevo"); ?></th>
              <th class="left"><?php esc_html_e("Name", "cluevo"); ?></th>
              <th><?php esc_html_e("Modules", "cluevo"); ?></th>
              <th><?php esc_html_e("Users", "cluevo"); ?></th>
              <th><?php esc_html_e("Attempts", "cluevo"); ?></th>
              <th>&#8960; <?php esc_html_e("Points", "cluevo"); ?></th>
              <th>&#8960; <?php esc_html_e("Points %", "cluevo"); ?></th>
              <th><?php esc_html_e("Completed", "cluevo"); ?></th>
              <th class="left"><?php esc_html_e("Best User", "cluevo"); ?></th>
              <th class="left"><?php esc_html_e("Worst User", "cluevo"); ?></th>
            </tr>
            <?php foreach ($items as $item) { ?>
              <tr>
                <td><?php echo esc_html($item->item_id); ?></td>
                <td class="left"><?php if (empty($item->module) || $item->module < 0) {
                                    esc_html_e($item->type, "cluevo");
                                  } else {
                                    esc_html_e("Module", "cluevo");
                                  } ?></td>
                <?php if (empty($item->module) || $item->module < 0) { ?>
                  <td class="left cluevo-table-filter" data-target="#item" data-id="<?php echo esc_html($item->item_id); ?>"><?php echo esc_html(mb_strimwidth($item->name, 0, 40, "...")); ?></td>
                <?php } else { ?>
                  <td class="left"><a href="<?php echo esc_url(admin_url("admin.php?page=$active_page&tab=" . CLUEVO_ADMIN_TAB_REPORTS_PROGRESS . "&module=$item->module_id"), ['http', 'https']); ?>"><?php echo esc_html(mb_strimwidth($item->name, 0, 20, "...")); ?></a></td>
                <?php } ?>
                <td><?php if (!empty($item->details->modules)) echo esc_html($item->details->modules); ?></td>
                <td><?php if (!empty($item->details->users)) echo esc_html($item->details->users); ?> / <?php echo esc_html($totalUsers); ?></td>
                <td><?php if (!empty($item->details->attempts)) echo esc_html($item->details->attempts); ?></td>
                <td><?php if (!empty($item->details->avg_score_raw)) echo esc_html(number_format($item->details->avg_score_raw, 2)); ?></td>
                <td><?php if (!empty($item->details->avg_score_scaled)) echo esc_html(number_format($item->details->avg_score_scaled * 100, 2)); ?>%</td>
                <td><?php if (!empty($item->details->completed_count)) echo esc_html($item->details->completed_count); ?></td>
                <td class="left"><?php if (!empty($item->details->best_user)) echo esc_html($item->details->best_user); ?></td>
                <td class="left"><?php if (!empty($item->details->worst_user)) echo ($item->details->worst_user != $item->details->best_user) ? esc_html($item->details->worst_user) : ""; ?></td>
              </tr>
            <?php } ?>
          </table>
        </form>
        <div class="cluevo-admin-table-pagination">
          <a class="cluevo-btn <?php if ($page < 0 || empty($page)) echo "disabled"; ?>" href="<?php echo esc_url(remove_query_arg("cur-page"), ['http', 'https']); ?>"><span class="dashicons dashicons-controls-skipback"></span></a>
          <a class="cluevo-btn <?php if ($page < 0 || empty($page)) echo "disabled"; ?>" href="<?php echo esc_url(add_query_arg("cur-page", "" . ($page - 1)), ['http', 'https']); ?>"><span class="dashicons dashicons-controls-back"></span></a>
          <div class="cluevo-select-page">
            Seite
            <select name="cur-page">
              <?php for ($i = 0; $i < $pagination["pages"]; $i++) { ?>
                <option value="<?php echo esc_attr($i); ?>" <?php if ($i == $page) echo "selected"; ?>><?php echo esc_html($i + 1); ?></option>
              <?php } ?>
            </select>
            von <?php echo esc_html($pagination["pages"]); ?>
          </div>
          <a class="cluevo-btn <?php if (empty($nextPage)) echo "disabled"; ?>" href="<?php echo esc_url(add_query_arg("cur-page", "$nextPage"), ['http', 'https']); ?>"><span class="dashicons dashicons-controls-forward"></span></a>
          <a class="cluevo-btn <?php if (empty($nextPage) && $nextPage < $pagination["pages"]) echo "disabled"; ?>" href="<?php echo esc_url(add_query_arg("cur-page", $pagination["pages"] - 1), ['http', 'https']); ?>"><span class="dashicons dashicons-controls-skipforward"></span></a>
        </div>
      <?php } else { ?>
        <div class="cluevo-admin-notice cluevo-notice-info">
          <p><?php esc_html_e("No reports found.", "cluevo"); ?></p>
        </div>
      <?php } ?>
    <?php }
  }

  function cluevo_render_reports_tab_progress()
  {

    $userId = (!empty($_GET["user"]) && is_numeric($_GET["user"])) ? (int)$_GET["user"] : null;
    $userName = (!empty($_GET["cluevo-autocomplete-input-user"])) ? $_GET["cluevo-autocomplete-input-user"] : null;
    $moduleId = (!empty($_GET["module"]) && is_numeric($_GET["module"])) ? (int)$_GET["module"] : null;
    $moduleName = (!empty($_GET["cluevo-autocomplete-input-module"])) ? $_GET["cluevo-autocomplete-input-module"] : null;
    $attemptId = (isset($_GET["attempt"]) && is_numeric($_GET["attempt"]) && (int)$_GET["attempt"] >= 0) ? (int)$_GET["attempt"] : null;
    $successStatus = (!empty($_GET["success-status"])) ? cluevo_strip_non_alpha($_GET["success-status"]) : null;
    $tags = (!empty($_GET["tags"])) ? sanitize_text_field($_GET["tags"]) : null;
    $completionStatus = (!empty($_GET["completion-status"])) ? cluevo_strip_non_alpha($_GET["completion-status"]) : null;
    $lessonStatus = (!empty($_GET["lesson-status"])) ? cluevo_strip_non_alpha_blank($_GET["lesson-status"]) : null;
    $page = (isset($_GET["cur-page"]) && is_numeric($_GET["cur-page"]) && (int)$_GET["cur-page"] >= 0) ? (int)$_GET["cur-page"] : 0;
    $perPage = 100;
    $pagination = cluevo_get_progress_pagination([
      "module_id" => $moduleId,
      "user_id" => $userId,
      "attempt_id" => $attemptId,
      "success_status" => $successStatus,
      "completion_status" => $completionStatus,
      "lesson_status" => $lessonStatus
    ], $perPage);

    $entries = cluevo_get_modules_progress_entries(
      ["user_id" => $userId,
      "module_id" => $moduleId,
      "attempt_id" => $attemptId,
      "module_id" => $moduleId,
      "success_status" => $successStatus,
      "completion_status" => $completionStatus, 
      "lesson_status" => $lessonStatus,
      "tags" => $tags
      ],
      $page, $perPage);
    $modules = cluevo_get_progress_filter_content("module_id", "m.module_name", ["user_id" => $userId, "attempt_id" => $attemptId, "module_id" => $moduleId]);
    $users = cluevo_get_progress_filter_content("user_id", "u.display_name", ["module_id" => $moduleId, "attempt_id" => $attemptId]);
    $attempts = cluevo_get_progress_filter_content("attempt_id", "attempt_id", ["user_id" => $userId, "module_id" => $moduleId, "module_id" => $moduleId]);
    if (!empty($attempts) && is_array($attempts)) {
      foreach ($attempts as $key => $value) {
        $attempts[$key]->label = (int)$value->value + 1;
      }
    }
    $successFilter = cluevo_get_progress_filter_content("success_status", "success_status", ["user_id" => $userId, "module_id" => $moduleId, "module_id" => $moduleId, "attempt_id" => $attemptId]);
    $completionFilter = cluevo_get_progress_filter_content("completion_status", "completion_status", ["user_id" => $userId, "module_id" => $moduleId, "module_id" => $moduleId, "attempt_id" => $attemptId]);
    $lessonFilter = cluevo_get_progress_filter_content("lesson_status", "lesson_status", ["user_id" => $userId, "module_id" => $moduleId, "module_id" => $moduleId, "attempt_id" => $attemptId]);
    $active_tab = (!empty($_GET["tab"]) && ctype_alpha($_GET["tab"])) ? cluevo_strip_non_alpha($_GET["tab"]) : CLUEVO_ADMIN_TAB_REPORTS_MAIN;
    $active_page = CLUEVO_ADMIN_PAGE_REPORTS;

    $nextPage = ($page + 1 < $pagination["pages"]) ? $page + 1 : null;
    $availableColumns = [
      "user" => [
        "name" => __("User", "cluevo"),
        "render" => function ($row) { ?>
      <td class="left cluevo-table-filter" data-target="#filter-user" data-id="<?php echo esc_attr($row->user_id); ?>"><?php echo esc_html($row->display_name); ?></td>
    <?php
        }, "render_filter" => function () use ($userName, $userId) { ?>
      <td>
        <div class="cluevo-autocomplete-container" data-name="user" data-type="users" data-initial="<?php echo esc_attr($userName); ?>" data-value="<?php echo $userId ? esc_attr((int)$userId) : ''; ?>"></div>
      </td>

    <?php }
      ],
      "email" => ["name" => __("E-Mail", "cluevo"), "render" => function ($row) { ?>

      <td class="left"><?php echo esc_html($row->user_email); ?></td>
    <?php },],
    "module" => [
      "name" => __("Module", "cluevo"),
      "render" => function ($row) { ?>
      <td class="left cluevo-table-filter" data-target="#filter-module" data-id="<?php echo esc_attr($row->module_id); ?>">
        <?php echo esc_html($row->module_name); ?>
      </td>

    <?php },
      "render_filter" => function () use ($moduleName, $moduleId) { ?>
      <td>
        <div class="cluevo-autocomplete-container" data-name="module" data-type="modules" data-min="0" data-initial="<?php echo esc_attr($moduleName); ?>" data-value="<?php echo $moduleId ? esc_attr((int)$moduleId) : ''; ?>"></div>
      </td>
    <?php },
    ],
    "tags" => [
      "name" => __("tags", "cluevo"),
      "render" => function ($row) { ?>
      <td class="left cluevo-table-filter" data-target="#filter-tags" data-id="<?php echo esc_attr($row->tags); ?>">
        <?php echo esc_html($row->tags); ?>
      </td>

    <?php },
      "render_filter" => function () use ($tags) { ?>
        <td><input type="text" name="tags" id="filter-tags" class="cluevo-filter-text" value="<?php echo esc_attr($tags); ?>" /></td>
    <?php },
    ],
      "attempt" => ["name" => __("Attempt", "cluevo"), "render" => function ($row) { ?>
      <td class="cluevo-table-filter" data-target="#filter-attempt" data-id="<?php echo esc_attr($row->attempt_id); ?>"><?php echo esc_html((int)$row->attempt_id + 1); ?></td>

    <?php }, "render_filter" => function () use ($attempts, $attemptId) { ?>
      <td><?php cluevo_render_admin_table_filter("attempt", __("all attempts", "cluevo"), -1, $attempts, $attemptId); ?></td>

    <?php },],
      "start" => ["name" => __("Start", "cluevo"), "render" => function ($row) { ?>
      <td><?php echo esc_html($row->date_started); ?></td>

    <?php },],
      "last_activity" => ["name" =>  __("Last Activity", "cluevo"), "render" => function ($row) { ?>
      <td><?php echo esc_html($row->date_modified); ?></td>

    <?php },],
      "min_pts" => ["name" => __("Min. Pts.", "cluevo"), "render" => function ($row) { ?>
      <td><?php echo esc_html(number_format($row->score_min, 2)); ?></td>

    <?php },],
      "max_pts" => ["name" => __("Max. Pts.", "cluevo"), "render" => function ($row) { ?>
      <td><?php echo esc_html(number_format($row->score_max, 2)); ?></td>

    <?php },],
      "points" => ["name" => __("Points", "cluevo"), "render" => function ($row) { ?>
      <td><?php echo esc_html(number_format($row->score_raw, 2)); ?></td>

    <?php },],
      "points_pct" => ["name" => __("Points %", "cluevo"), "render" => function ($row) { ?>
      <td><?php echo esc_html(number_format($row->score_scaled * 100, 2)); ?>%</td>

    <?php },],
      "completion_status" => ["name" => __("Completion Status", "cluevo"), "render" => function ($row) { ?>
      <td class="status <?php echo esc_attr($row->completion_status); ?> <?php echo ($row->scorm_version == "1.2") ? "invalid" : ""; ?> cluevo-table-filter" data-target="#filter-completion-status" data-id="<?php echo esc_attr($row->completion_status); ?>"><?php echo esc_html($row->completion_status); ?></td>

    <?php }, "render_filter" => function () use ($completionFilter, $completionStatus) { ?>
      <td><?php cluevo_render_admin_table_filter("completion-status", __("all", "cluevo"), 0, $completionFilter, $completionStatus); ?></td>

    <?php },],
      "success_status" => ["name" => __("Success Status", "cluevo"), "render" => function ($row) { ?>
      <td class="status <?php echo esc_attr($row->success_status); ?> <?php echo ($row->scorm_version == "1.2") ? "invalid" : ""; ?> cluevo-table-filter" data-target="#filter-success-status" data-id="<?php echo esc_attr($row->success_status); ?>"><?php echo esc_html($row->success_status); ?></td>

    <?php }, "render_filter" => function () use ($successFilter, $successStatus) { ?>
      <td><?php cluevo_render_admin_table_filter("success-status", __("all", "cluevo"), 0, $successFilter, $successStatus); ?></td>
    <?php },],
      "lesson_status" => ["name" => __("Lesson Status (SCORM 1.2)", "cluevo"), "render" => function ($row) { ?>
      <td class="status <?php echo esc_attr($row->lesson_status); ?> <?php echo ($row->scorm_version != "1.2") ? "invalid" : ""; ?> cluevo-table-filter" data-target="#filter-lesson-status" data-id="<?php echo esc_attr($row->lesson_status); ?>"><?php echo esc_html($row->lesson_status); ?></td>

    <?php }, "render_filter" => function ()  use ($lessonFilter, $lessonStatus) { ?>
      <td><?php cluevo_render_admin_table_filter("lesson-status", __("all", "cluevo"), 0, $lessonFilter, $lessonStatus); ?></td>

    <?php },],
      "credit" => ["name" => __("Credit", "cluevo"), "render" => function ($row) { ?>
      <td class="credit-status toggle-credit <?php echo esc_attr($row->credit); ?>">
        <div class="cluevo-btn"><?php echo ($row->credit == "credit") ? "🗸" : "🞨"; ?></div>
      </td>

    <?php },],
    ];
    if (!empty($_POST["set_columns"])) {
      $disableColumns = [];
      if (!empty($_POST["columns"])) {
        foreach ($availableColumns as $key => $column) {
          if (!in_array($key, $_POST["columns"])) {
            $disableColumns[] = sanitize_key($key);
          }
        }
      }
      update_option("cluevo_progress_disabled_columns", $disableColumns);
    }
    wp_register_script(
      'cluevo-admin-table-filter',
      plugins_url(
        '/js/admin-table-filter.js',
        plugin_dir_path(__FILE__)
      ),
      ["cluevo-admin-common"],
      CLUEVO_VERSION,
      true
    );
    wp_enqueue_script("cluevo-admin-table-filter");
    wp_register_script(
      "vue-js",
      plugins_url("/js/vue.min.js", plugin_dir_path(__FILE__)),
      "",
      CLUEVO_VERSION,
      false
    );
    wp_enqueue_script('vue-js');
    wp_register_script(
      'cluevo-user-autocomplete',
      plugins_url(
        '/js/user-selector.vue.js',
        plugin_dir_path(__FILE__)
      ),
      ["vue-js"],
      CLUEVO_VERSION,
      false
    );
    wp_enqueue_script("cluevo-user-autocomplete");
    wp_register_script(
      'cluevo-progress-editor',
      plugins_url(
        '/js/progress-editor.js',
        plugin_dir_path(__FILE__)
      ),
      ["vue-js"],
      CLUEVO_VERSION,
      true
    );
    wp_localize_script(
      'cluevo-progress-editor',
      'cluevoProgressSettings',
      [
        'ajax_url' => admin_url('admin-ajax.php'),
        'nonce' => wp_create_nonce('cluevo-progress---'),
      ]
    );
    wp_enqueue_script("cluevo-progress-editor");
    $disabledColumns = get_option('cluevo_progress_disabled_columns', []);
    if (!is_array($disabledColumns)) {
      $disabledColumns = [];
      update_option("cluevo_progress_disabled_columns", $disabledColumns);
    }
    ?>
    <h1><?php echo esc_html($pagination["items"]) . " " . esc_html__("records found", "cluevo"); ?></h1>
    <form action="<?php echo esc_url(remove_query_arg('cur-page')); ?>" method="post">
      <input type="hidden" name="page" value="<?php echo esc_attr($active_page); ?>" />
      <input type="hidden" name="tab" value="<?php echo esc_attr($active_tab); ?>" />
      <details class="cluevo-column-selection-container">
        <summary><?php echo esc_html(sprintf(__("%s/%s Columns", "cluevo"), count($availableColumns) - count($disabledColumns), count($availableColumns))); ?></summary>
        <ul>
          <?php foreach ($availableColumns as $key => $col) { ?>
            <label>
              <input type="checkbox" name="columns[]" value="<?php echo esc_attr($key); ?>" <?php if (is_array($disabledColumns) && !in_array($key, $disabledColumns)) echo "checked"; ?> /><?php echo esc_html($col["name"]); ?>
            </label>
          <?php } ?>
          <button type="submit" name="set_columns" value="set_columns" class="cluevo-btn"><?php esc_html_e("Submit", "cluevo"); ?></button>
      </details>
    </form>
    <form action="<?php echo esc_url(remove_query_arg('cur-page')); ?>">
      <input type="hidden" name="page" value="<?php echo esc_attr($active_page); ?>" />
      <input type="hidden" name="tab" value="<?php echo esc_attr($active_tab); ?>" />
      <table class="cluevo-filtered-table cluevo-admin-table">
        <tr class="filter-row">
          <?php foreach ($availableColumns as $key => $col) { ?>
            <?php if (is_array($disabledColumns) && !in_array($key, $disabledColumns)) { ?>
              <?php if (!empty($col["render_filter"]) && is_callable($col["render_filter"])) {
                $col["render_filter"]();
              } else { ?> <td></td> <?php } ?>
            <?php } ?>
          <?php } ?>
          <td colspan="2">
            <div class="cluevo-buttons">
              <div class="cluevo-btn auto cluevo-btn-secondary cluevo-reset-filters"><?php esc_attr_e("Reset", "cluevo"); ?></div>
              <a href="<?php echo add_query_arg('cluevo-export', 'progress'); ?>" target="_blank" class="cluevo-btn auto cluevo-btn-secondary cluevo-export-data"><?php esc_attr_e("Export", "cluevo"); ?></a>
            </div>
          </td>
        </tr>
        <tr>
          <?php foreach ($availableColumns as $key => $col) { ?>
            <?php if (is_array($disabledColumns) && !in_array($key, $disabledColumns)) { ?>
              <th class="<?php echo in_array($key, ["user", "email", "module", "tags"]) ? "left" : ""; ?>">
                <?php echo esc_html($col["name"]); ?>
              </th>
            <?php } ?>
          <?php } ?>
          <th><?php esc_html_e("Tools", "cluevo"); ?></th>
        </tr>
        <?php if (!empty($entries)) { ?>
          <?php foreach ($entries as $row) { ?>
            <tr data-user-id="<?php echo esc_attr($row->user_id); ?>" data-module-id="<?php echo esc_attr($row->module_id); ?>" data-attempt-id="<?php echo esc_attr($row->attempt_id); ?>">
              <?php foreach ($availableColumns as $key => $col) { ?>
                <?php if (is_array($disabledColumns) && !in_array($key, $disabledColumns)) { ?>
                  <?php if (is_callable($col["render"])) $col["render"]($row); ?>
                <?php } ?>
              <?php } ?>
              <td>
                <div class="cluevo-buttons">
                  <div class="cluevo-btn cluevo-edit-progress"><span class="dashicons dashicons-edit"></span></div>
                  <?php if (strpos($row->type_name, 'scorm') === false) { ?>
                    <p class="cluevo-btn disabled" title="<?php esc_attr_e("No SCORM parameters available for this module type", "cluevo"); ?>" href="<?php echo esc_url(admin_url("admin.php?page=" . CLUEVO_ADMIN_PAGE_REPORTS . "&tab=" . CLUEVO_ADMIN_TAB_REPORTS_SCORM_PARMS . "&module=" . $row->module_id), ['http', 'https']); ?>"><span class="dashicons dashicons-admin-settings"></span></p>
                  <?php } else { ?>
                    <a class="cluevo-btn" title="<?php esc_attr_e("Browse SCORM parameters", "cluevo"); ?>" href="<?php echo esc_url(admin_url("admin.php?page=" . CLUEVO_ADMIN_PAGE_REPORTS . "&tab=" . CLUEVO_ADMIN_TAB_REPORTS_SCORM_PARMS . "&module=" . $row->module_id), ['http', 'https']); ?>"><span class="dashicons dashicons-admin-settings"></span></a>
                  <?php } ?>
                </div>
              </td>
            </tr>
          <?php } ?>
      </table>
    <?php } else { ?>
      </table>
      <div class="cluevo-admin-notice cluevo-notice-info">
        <p><?php esc_html_e("No progress records found.", "cluevo"); ?></p>
      </div>
    <?php } ?>
    </form>
    <?php if (!empty($entries)) { ?>
      <div class="cluevo-admin-table-pagination">
        <a class="cluevo-btn <?php if ($page < 0 || empty($page)) echo "disabled"; ?>" href="<?php echo esc_url(remove_query_arg("cur-page"), ['http', 'https']); ?>"><span class="dashicons dashicons-controls-skipback"></span></a>
        <a class="cluevo-btn <?php if ($page < 0 || empty($page)) echo "disabled"; ?>" href="<?php echo esc_url(add_query_arg("cur-page", "" . ($page - 1)), ['http', 'https']); ?>"><span class="dashicons dashicons-controls-back"></span></a>
        <div class="cluevo-select-page">
          Seite
          <select name="cur-page">
            <?php for ($i = 0; $i < $pagination["pages"]; $i++) { ?>
              <option value="<?php echo esc_attr($i); ?>" <?php if ($i == $page) echo "selected"; ?>><?php echo esc_html($i + 1); ?></option>
            <?php } ?>
          </select>
          von <?php echo esc_html($pagination["pages"]); ?>
        </div>
        <a class="cluevo-btn <?php if (empty($nextPage)) echo "disabled"; ?>" href="<?php echo esc_url(add_query_arg("cur-page", "$nextPage"), ['http', 'https']); ?>"><span class="dashicons dashicons-controls-forward"></span></a>
        <a class="cluevo-btn <?php if (empty($nextPage) && $nextPage < $pagination["pages"]) echo "disabled"; ?>" href="<?php echo esc_url(add_query_arg("cur-page", $pagination["pages"] - 1), ['http', 'https']); ?>"><span class="dashicons dashicons-controls-skipforward"></span></a>
      </div>
      <div id="cluevo-progress-editor"></div>
      <script type="text/x-template" id="cluevo-progress-editor-template">
        <transition name="modal">
      <div v-if="!closed" class="modal-mask" @click.self="close">
        <div class="modal-wrapper">
          <div class="modal-container">
            <div class="modal-header">
              <h3><?php esc_html_e("Edit Progress Entry", "cluevo"); ?></h3>
              <button class="close" @click="close"><span class="dashicons dashicons-no-alt"></span></button>
            </div>
            <div class="modal-body">
              <form>
                <div class="cluevo-form-group">
                  <label><?php esc_html_e("Min. Pts.", "cluevo"); ?>
                    <input v-model="minPoints" type="number" name="cluevo-progress-min-pts" />
                  </label>
                </div>
                <div class="cluevo-form-group">
                  <label><?php esc_html_e("Max. Pts.", "cluevo"); ?>
                    <input v-model="maxPoints" type="number" name="cluevo-progress-max-pts" />
                  </label>
                </div>
                <div class="cluevo-form-group">
                  <label><?php esc_html_e("Points", "cluevo"); ?>
                    <input v-model="points" type="number" name="cluevo-progress-pts" />
                  </label>
                </div>
                <div class="cluevo-form-group">
                  <label><?php esc_html_e("Points %", "cluevo"); ?>
                    <input v-model="pointsPct" type="number" name="cluevo-progress-pts-pct" />
                  </label>
                </div>
                <div class="cluevo-form-group">
                  <label><?php esc_html_e("Completion Status", "cluevo"); ?>
                    <select v-model="completionStatus" name="cluevo-progress-completion-status">
                      <option value=""></option>
                      <option value="completed"><?php esc_html_e("Completed", "cluevo"); ?></option>
                      <option value="incomplete"><?php esc_html_e("Incomplete", "cluevo"); ?></option>
                      <option value="not attempted"><?php esc_html_e("Not Attempted", "cluevo"); ?></option>
                      <option value="unknown"><?php esc_html_e("Unknown", "cluevo"); ?></option>
                    </select>
                  </label>
                </div>
                <div class="cluevo-form-group">
                  <label><?php esc_html_e("Success Status", "cluevo"); ?>
                    <select v-model="successStatus" name="cluevo-progress-success-status">
                      <option value=""></option>
                      <option value="passed"><?php esc_html_e("Passed", "cluevo"); ?></option>
                      <option value="failed"><?php esc_html_e("Failed", "cluevo"); ?></option>
                      <option value="unknown"><?php esc_html_e("Unknown", "cluevo"); ?></option>
                    </select>
                  </label>
                </div>
                <div class="cluevo-form-group">
                  <label><?php esc_html_e("Lesson Status (SCORM 1.2)", "cluevo"); ?>
                    <select v-model="lessonStatus" name="cluevo-progress-lesson-status">
                      <option value=""></option>
                      <option value="passed"><?php esc_html_e("Passed", "cluevo"); ?></option>
                      <option value="completed"><?php esc_html_e("Completed", "cluevo"); ?></option>
                      <option value="failed"><?php esc_html_e("Failed", "cluevo"); ?></option>
                      <option value="incomplete"><?php esc_html_e("Incomplete", "cluevo"); ?></option>
                      <option value="browsed"><?php esc_html_e("browsed", "cluevo"); ?></option>
                      <option value="not attempted"><?php esc_html_e("Not Attempted", "cluevo"); ?></option>
                    </select>
                  </label>
                </div>
                <div class="cluevo-form-group">
                  <label class="cluevo-horizontal">
                    <input v-model="credit" type="checkbox" name="cluevo-progress-credit" :value="true" />
                    <?php esc_html_e("credit", "cluevo"); ?>
                  </label>
                </div>
                <div class="cluevo-buttons">
                  <div class="cluevo-btn primary" @click="save"><?php esc_html_e("Save", "cluevo"); ?></div>
                  <div class="cluevo-btn" @click="close"><?php esc_html_e("Cancel", "cluevo"); ?></div>
                </div>
                <div v-if="error" class="cluevo-notice cluevo-notice-error"><?php esc_html_e("Failed to update progress entry", "cluevo"); ?></div>
              </form>
            </div>
          </div>
        </div>
      </div>
    </transition>
      </script>
    <?php
    }
  }

  function cluevo_render_reports_tab_scorm()
  {
    $userId = (!empty($_GET["user"]) && is_numeric($_GET["user"])) ? (int)$_GET["user"] : null;
    $userName = (!empty($_GET["cluevo-autocomplete-input-user"])) ? sanitize_text_field($_GET["cluevo-autocomplete-input-user"]) : null;
    $moduleId = (!empty($_GET["module"]) && is_numeric($_GET["module"])) ? (int)$_GET["module"] : null;
    $moduleName = (!empty($_GET["cluevo-autocomplete-input-module"])) ? sanitize_text_field($_GET["cluevo-autocomplete-input-module"]) : null;
    $attemptId = (isset($_GET["attempt"]) && is_numeric($_GET["attempt"]) && (int)$_GET["attempt"] >= 0) ? (int)$_GET["attempt"] : null;
    $parameter = (!empty($_GET["parameter"])) ? cluevo_strip_non_scorm_parm_chars($_GET["parameter"]) : null;
    $page = (isset($_GET["cur-page"]) && is_numeric($_GET["cur-page"]) && (int)$_GET["cur-page"] >= 0) ? (int)$_GET["cur-page"] : 0;
    $perPage = 100;

    $users = cluevo_get_parameters_filter_content("user_id", "u.display_name", ["module_id" => $moduleId, "parameter" => $parameter, "attempt_id" => $attemptId]);
    $modules = cluevo_get_parameters_filter_content("module_id", "m.module_name", ["user_id" => $userId, "parameter" => $parameter, "attempt_id" => $attemptId]);
    $attempts = cluevo_get_parameters_filter_content("attempt_id", "attempt_id", ["user_id" => $userId, "parameter" => $parameter, "module_id" => $moduleId]);
    if (!empty($attempts) && is_array($attempts)) {
      foreach ($attempts as $key => $value) {
        $attempts[$key]->label = (int)$value->value + 1;
      }
    }
    $availableParms = cluevo_get_parameters_filter_content("parameter", "parameter", ["module_id" => $moduleId, "user_id" => $userId, "attempt_id" => $attemptId]);
    $parameters = cluevo_get_parameters(["module_id" => $moduleId, "user_id" => $userId, "attempt_id" => $attemptId, "parameter" => $parameter], $page, $perPage);
    $pagination = cluevo_get_parameter_pagination(["module_id" => $moduleId, "user_id" => $userId, "attempt_id" => $attemptId, "parameter" => $parameter], $perPage);
    $nextPage = ($page + 1 < $pagination["pages"]) ? $page + 1 : null;
    $active_tab = (!empty($_GET["tab"]) && ctype_alpha($_GET["tab"])) ? cluevo_strip_non_alpha($_GET["tab"]) : CLUEVO_ADMIN_TAB_REPORTS_MAIN;
    $active_page = CLUEVO_ADMIN_PAGE_REPORTS;

    wp_register_script('cluevo-admin-table-filter', plugins_url('/js/admin-table-filter.js', __DIR__), array(), CLUEVO_VERSION, true);
    wp_enqueue_script("cluevo-admin-table-filter");
    wp_register_script(
      "vue-js",
      plugins_url("/js/vue.min.js", plugin_dir_path(__FILE__)),
      "",
      CLUEVO_VERSION,
      false
    );
    wp_enqueue_script('vue-js');
    wp_register_script(
      'cluevo-user-autocomplete',
      plugins_url(
        '/js/user-selector.vue.js',
        plugin_dir_path(__FILE__)
      ),
      ["vue-js"],
      CLUEVO_VERSION,
      false
    );
    wp_enqueue_script("cluevo-user-autocomplete");
    ?>
    <h1><?php echo esc_html($pagination["items"]) . " " . esc_html__("records found", "cluevo"); ?></h1>
    <div class="cluevo-module-progress-filter-container">
    </div>

    <form action="<?php echo esc_url(remove_query_arg('cur-page')); ?>">
      <input type="hidden" name="page" value="<?php echo esc_attr($active_page); ?>" />
      <input type="hidden" name="tab" value="<?php echo esc_attr($active_tab); ?>" />
      <table class="cluevo-admin-table cluevo-filtered-table">
        <tr class="filter-row">
          <td></td>
          <td>
            <div class="cluevo-autocomplete-container" data-name="module" data-type="modules" data-min="0" data-initial="<?php echo esc_attr($moduleName); ?>" data-value="<?php echo $moduleId ? esc_attr((int)$moduleId) : ''; ?>"></div>
          </td>
          <td>
            <div class="cluevo-autocomplete-container" data-name="user" data-type="users" data-initial="<?php echo esc_attr($userName); ?>" data-value="<?php echo $userId ? esc_attr((int)$userId) : ''; ?>"></div>
          </td>
          <td></td>
          <td><?php cluevo_render_admin_table_filter("attempt", __("all attempts", "cluevo"), -1, $attempts, $attemptId); ?></td>
          <td><?php cluevo_render_admin_table_filter("parameter", __("all parameters", "cluevo"), 0, $availableParms, $parameter); ?></td>
          <td></td>
          <td></td>
          <td>
            <div class="cluevo-buttons">
              <div class="cluevo-btn auto cluevo-btn-secondary cluevo-reset-filters"><?php esc_attr_e("Reset Filters", "cluevo"); ?></div>
              <a href="<?php echo add_query_arg('cluevo-export', 'scorm'); ?>" target="_blank" class="cluevo-btn auto cluevo-btn-secondary cluevo-export-data"><?php esc_attr_e("Export", "cluevo"); ?></a>
            </div>
          </td>
        </tr>
        <tr>
          <th>#</th>
          <th class="left"><?php esc_html_e("Module", "cluevo"); ?></th>
          <th class="left"><?php esc_html_e("User", "cluevo"); ?></th>
          <th class="left"><?php esc_html_e("E-Mail", "cluevo"); ?></th>
          <th><?php esc_html_e("Attempt", "cluevo"); ?></th>
          <th class="left"><?php esc_html_e("Parameter", "cluevo"); ?></th>
          <th><?php esc_html_e("Worth", "cluevo"); ?></th>
          <th><?php esc_html_e("Created", "cluevo"); ?></th>
          <th><?php esc_html_e("Modified", "cluevo"); ?></th>
          <!-- <th><?php esc_html_e("Tools", "cluevo"); ?></th> -->
        </tr>
        <?php if (!empty($parameters)) { ?>
          <?php foreach ($parameters as $p) { ?>
            <tr>
              <td><?php echo esc_html($p->id); ?></td>
              <td class="left cluevo-table-filter" data-target="#filter-module" data-id="<?php echo esc_attr($p->module_id); ?>">
                <?php if (strlen($p->module_name) > 20) { ?>
                  <?php echo esc_html(mb_strimwidth($p->module_name, 0, 20, "...")); ?>
                <?php } else { ?>
                  <?php echo esc_html($p->module_name); ?>
                <?php } ?>
              </td>
              <td class="left cluevo-table-filter" data-target="#filter-user" data-id="<?php echo esc_attr($p->user_id); ?>"><?php echo esc_html($p->display_name); ?></td>
              <td class="left"><?php echo esc_html($p->user_email); ?></td>
              <td class="cluevo-table-filter" data-target="#filter-attempt" data-id="<?php echo esc_attr($p->attempt_id); ?>"><?php echo esc_html((int)$p->attempt_id + 1); ?></td>
              <td class="left cluevo-table-filter" data-target="#filter-parameter" data-id="<?php echo esc_attr($p->parameter); ?>"><?php echo esc_html($p->parameter); ?></td>
              <td>
                <?php if (strlen($p->value) > 20) { ?>
                  <?php echo esc_html(mb_strimwidth($p->value, 0, 20, "...")); ?>
                <?php } else { ?>
                  <?php echo esc_html($p->value); ?>
                <?php } ?>
              </td>
              <td><?php echo esc_html($p->date_added); ?></td>
              <td><?php echo esc_html($p->date_modified); ?></td>
              <!-- <td></td> -->
            </tr>
          <?php } ?>
      </table>
    <?php } else { ?>
      </table>
      <div class="cluevo-admin-notice cluevo-notice-info">
        <p><?php esc_html_e("No SCORM parameters found.", "cluevo"); ?></p>
      </div>
    <?php } ?>
    </form>
    <?php if (!empty($parameters)) { ?>
      <div class="cluevo-admin-table-pagination">
        <a class="cluevo-btn <?php if ($page < 0 || empty($page)) echo "disabled"; ?>" href="<?php echo esc_url(remove_query_arg("cur-page"), ['http', 'https']); ?>"><span class="dashicons dashicons-controls-skipback"></span></a>
        <a class="cluevo-btn <?php if ($page < 0 || empty($page)) echo "disabled"; ?>" href="<?php echo esc_url(add_query_arg("cur-page", "" . ($page - 1)), ['http', 'https']); ?>"><span class="dashicons dashicons-controls-back"></span></a>
        <div class="cluevo-select-page">
          <?php esc_html_e("Page", "cluevo"); ?>
          <select name="cur-page">
            <?php for ($i = 0; $i < $pagination["pages"]; $i++) { ?>
              <option value="<?php echo esc_attr($i); ?>" <?php if ($i == $page) echo "selected"; ?>><?php echo esc_html($i + 1); ?></option>
            <?php } ?>
          </select>
          <?php echo esc_html__("of", "cluevo") . " " . esc_html($pagination["pages"]); ?>
        </div>
        <a class="cluevo-btn <?php if (empty($nextPage)) echo "disabled"; ?>" href="<?php echo esc_url(add_query_arg("cur-page", "$nextPage"), ['http', 'https']); ?>"><span class="dashicons dashicons-controls-forward"></span></a>
        <a class="cluevo-btn <?php if (empty($nextPage) && $nextPage < $pagination["pages"]) echo "disabled"; ?>" href="<?php echo esc_url(add_query_arg("cur-page", $pagination["pages"] - 1), ['http', 'https']); ?>"><span class="dashicons dashicons-controls-skipforward"></span></a>
      </div>
    <?php } ?>
  <?php
  }

  add_action('cluevo_output_reports_tabs', function () {
    $active_tab = (!empty($_GET["tab"]) && ctype_alpha($_GET["tab"])) ? cluevo_strip_non_alphanum_dash($_GET["tab"]) : CLUEVO_ADMIN_TAB_REPORTS_MAIN;
  ?>
    <a href="<?php echo esc_url(admin_url("admin.php?page=" . CLUEVO_ADMIN_PAGE_REPORTS . "&tab=" . CLUEVO_ADMIN_TAB_REPORTS_MAIN), ['http', 'https']); ?>" class="nav-tab <?php echo ($active_tab == CLUEVO_ADMIN_TAB_REPORTS_MAIN) ? 'nav-tab-active' : ''; ?>"><?php esc_html_e("Reports", "cluevo"); ?></a>
  <?php });

  add_action('cluevo_output_reports_tabs', function () {
    $active_tab = (!empty($_GET["tab"]) && ctype_alpha($_GET["tab"])) ? cluevo_strip_non_alphanum_dash($_GET["tab"]) : CLUEVO_ADMIN_TAB_REPORTS_MAIN;
  ?>
    <a href="<?php echo esc_url(admin_url("admin.php?page=" . CLUEVO_ADMIN_PAGE_REPORTS . "&tab=" . CLUEVO_ADMIN_TAB_REPORTS_PROGRESS), ['http', 'https']); ?>" class="nav-tab <?php echo ($active_tab == CLUEVO_ADMIN_TAB_REPORTS_PROGRESS) ? 'nav-tab-active' : ''; ?>"><?php esc_html_e("Progress", "cluevo"); ?></a>
  <?php });
  add_action('cluevo_output_reports_tabs', function () {
    $active_tab = (!empty($_GET["tab"]) && ctype_alpha($_GET["tab"])) ? cluevo_strip_non_alphanum_dash($_GET["tab"]) : CLUEVO_ADMIN_TAB_REPORTS_MAIN;
  ?>
    <a href="<?php echo esc_url(admin_url("admin.php?page=" . CLUEVO_ADMIN_PAGE_REPORTS . "&tab=" . CLUEVO_ADMIN_TAB_REPORTS_SCORM_PARMS), ['http', 'https']); ?>" class="nav-tab <?php echo ($active_tab == CLUEVO_ADMIN_TAB_REPORTS_SCORM_PARMS) ? 'nav-tab-active' : ''; ?>"><?php esc_html_e("SCORM Parameters", "cluevo"); ?></a>
  <?php });

  add_action('cluevo_render_report_tab_scorm', 'cluevo_render_reports_tab_scorm');
  add_action('cluevo_render_report_tab_progress', 'cluevo_render_reports_tab_progress');
  add_action('cluevo_render_report_tab_reports', 'cluevo_render_reports_tab_reports');
  ?>
